(function(global) {
  'use strict'
  function Nlib() { }

  // **************************************************************************
  // Date
  // **************************************************************************
  (function() {
    var dateFormats = [
      'yyyy-mm-ddthhmmss.fff'
    , 'yyyy-mm-dd hhmmss.fff'
    , 'mm/dd/yyyy hhmmss.fff'
    , 'dd/mm/yyyy hhmmss.fff'
    , 'yyyymmddhhmmssfff'
    , 'yyyy-mm-ddthhmmss'
    , 'yyyy-mm-dd hhmmss'
    , 'mm/dd/yyyy hhmmss'
    , 'dd/mm/yyyy hhmmss'
    , 'yyyymmddhhmmss'
    , 'yyyy-mm-ddthhmm'
    , 'yyyy-mm-dd hhmm'
    , 'mm/dd/yyyy hhmm'
    , 'dd/mm/yyyy hhmm'
    , 'yyyymmddhhmm'
    , 'yyyy-mm-ddthh'
    , 'yyyy-mm-dd hh'
    , 'mm/dd/yyyy hh'
    , 'dd/mm/yyyy hh'
    , 'yyyymmddhh'
    , 'yyyy-mm-dd'
    , 'yyyymmdd'
    , 'yyyy-mm'
    , 'yyyymm'
    , 'yyyy'
    , 'mm/dd/yyyy'
    , 'dd/mm/yyyy'
    ];
    Nlib.prototype.parseDate = function(str, format) {
      if (!str || typeof str != 'string') {
        return undefined;
      }
      if (!format) {
        for (var i = 0; i < dateFormats.length; i++) {
          var date = this.parseDate(str, dateFormats[i]);
          if (date) {
            return date;
          }
        }
        return undefined;
      }
      switch(format) {
        case 'yyyy-mm-ddthh:mm:ss.fff':
        case 'yyyy-mm-dd hh:mm:ss.fff':
        case 'mm/dd/yyyy hh:mm:ss.fff':
        case 'dd/mm/yyyy hh:mm:ss.fff':
          if (str.length < 23) return undefined;
          fff = parseInt(str.substring(20, 23), 10);
          if (isNaN(fff) || fff < 0) return undefined;
          format = format.substring(0, 19);
          break;
        case 'yyyymmddhhmmssfff':
          if (str.length < 17) return undefined;
          fff = parseInt(str.substring(14, 17), 10);
          if (isNaN(fff) || fff < 0) return undefined;
          format = format.substring(0, 14);
          break;
        // seconds
        case 'yyyy-mm-ddthh:mm:ss':
        case 'yyyy-mm-dd hh:mm:ss':
        case 'mm/dd/yyyy hh:mm:ss':
        case 'dd/mm/yyyy hh:mm:ss':
          if (str.length < 19) return undefined;
          ss = parseInt(str.substring(17, 19), 10);
          if (isNaN(ss) || ss < 0 || ss > 59) return undefined;
          format = format.substring(0, 16);
          break;
        case 'yyyymmddhhmmss':
          if (str.length < 14) return undefined;
          ss = parseInt(str.substring(12, 14), 10);
          if (isNaN(ss) || ss < 0 || ss > 59) return undefined;
          format = format.substring(0, 12);
          break;
        // minutes
        case 'yyyy-mm-ddthh:mm':
        case 'yyyy-mm-dd hh:mm':
        case 'mm/dd/yyyy hh:mm':
        case 'dd/mm/yyyy hh:mm':
          if (str.length < 16) return undefined;
          mm = parseInt(str.substring(14, 16), 10);
          if (isNaN(mm) || mm < 0 || mm > 59) return undefined;
          format = format.substring(0, 13);
          break;
        case 'yyyymmddhhmm':
          if (str.length < 12) return undefined;
          mm = parseInt(str.substring(10, 12), 10);
          if (isNaN(mm) || mm < 0 || mm > 59) return undefined;
          format = format.substring(0, 10);
          break;
        // hours
        case 'yyyy-mm-ddthh':
        case 'yyyy-mm-dd hh':
        case 'mm/dd/yyyy hh':
        case 'dd/mm/yyyy hh':
          if (str.length < 13) return undefined;
          hh = parseInt(str.substring(11, 13), 10);
          if (isNaN(hh) || hh < 0 || hh > 59) return undefined;
          format = format.substring(0, 10);
          break;
        case 'yyyymmddhh':
          if (str.length < 10) return undefined;
          mm = parseInt(str.substring(8, 10), 10);
          if (isNaN(mm) || mm < 0 || mm > 59) return undefined;
          format = format.substring(0, 8);
          break;
        // day
        case 'yyyy-mm-dd':
          if (str.length < 10) return undefined;
          dd = parseInt(str.substring(8, 10), 10);
          if (isNaN(dd) || dd < 1 || dd > 31) return undefined;
          format = format.substring(0, 7);
          break;
        case 'yyyymmdd':
          if (str.length < 8) return undefined;
          dd = parseInt(str.substring(6, 8), 10);
          if (isNaN(dd) || dd < 1 || dd > 31) return undefined;
          format = format.substring(0, 6);
          break;
        // month
        case 'yyyy-mm':
          if (str.length < 7) return undefined;
          MM = parseInt(str.substring(5, 7), 10);
          if (isNaN(MM) || MM < 1 || MM > 12) return undefined;
          MM = MM - 1;
          format = format.substring(0, 4);
          break;
        case 'yyyymm':
          if (str.length < 6) return undefined;
          MM = parseInt(str.substring(4, 6), 10);
          if (isNaN(MM) || MM < 1 || MM > 12) return undefined;
          MM = MM - 1;
          format = format.substring(0, 4);
          break;
        // year
        case 'yyyy':
          if (str.length < 4) return undefined;
          yyyy = parseInt(str, 10);
          if (isNaN(yyyy) || yyyy < 0 ) return undefined;
          return new Date(yyyy, MM, dd, hh, mm, ss, fff);
        //other
        case 'mm/dd/yyyy':
          if (str.length < 10) return undefined;
          dd = parseInt(str.substring(3, 5), 10);
          if (isNaN(dd) || dd < 1 || dd > 31) return undefined;
          MM = parseInt(str.substring(0, 2), 10);
          if (isNaN(MM) || MM < 1 || MM > 12) return undefined;
          MM = MM - 1;
          yyyy = parseInt(str.substring(6, 10), 10);
          if (isNaN(yyyy) || yyyy < 0 ) return undefined;
          return new Date(yyyy, MM, dd, hh, mm, ss, fff);
        case 'dd/mm/yyyy':
          if (str.length < 10) return undefined;
          dd = parseInt(str.substring(0, 2), 10);
          if (isNaN(dd) || dd < 1 || dd > 31) return undefined;
          MM = parseInt(str.substring(3, 5), 10);
          if (isNaN(MM) || MM < 1 || MM > 12) return undefined;
          MM = MM - 1;
          yyyy = parseInt(str.substring(6, 10), 10);
          if (isNaN(yyyy) || yyyy < 0 ) return undefined;
          return new Date(yyyy, MM, dd, hh, mm, ss, fff);
        default:
          return undefined;
      }
    };
    Nlib.prototype.stringifyDate = function(date, format) {
      if (!date || typeof date != 'object' || Object.prototype.toString.call(date) != '[object Date]') {
        return '';
      }
      if (!format || typeof format != 'string') {
        return date.toString();
      }
      var yyyy = date.getFullYear() + '';
      var MM = date.getMonth() + 1;
      MM = ((MM < 10) ? '0' : '') + MM;
      var dd = date.getDate();
      dd = ((dd < 10) ? '0' : '') + dd;
      var hh = date.getHours();
      hh = ((hh < 10) ? '0' : '') + hh;
      var mm = date.getMinutes();
      mm = ((mm < 10) ? '0' : '') + mm;
      var ss = date.getSeconds();
      ss = ((ss < 10) ? '0' : '') + ss;
      var fff = date.getMilliseconds();
      fff = ((fff < 10) ? '00' : ((fff < 100) ? '0' : '')) + fff;
      
      format = format.toLowerCase();
      switch (format) {
        case 'yyyymmddhhmm':
          return yyyy+MM+dd+hh+mm;
        case 'yyyymmdd':
          return yyyy+MM+dd;
        case 'yyyy-mm-ddthh:mm:ss.fff':
          return yyyy+'-'+MM+'-'+dd+'T'+hh+':'+mm+':'+ss+'.'+fff;
        case 'yyyy-mm-dd hh:mm:ss.fff':
          return yyyy+'-'+MM+'-'+dd+' '+hh+':'+mm+':'+ss+'.'+fff;
        case 'mm/dd/yyyy hh:mm:ss.fff':
          return MM+'/'+dd+'/'+yyyy+' '+hh+':'+mm+':'+ss+'.'+fff;
        case 'yyyy-mm-ddthh:mm:ss':
          return yyyy+'-'+MM+'-'+dd+'T'+hh+':'+mm+':'+ss;
        case 'yyyy-mm-dd hh:mm:ss':
          return yyyy+'-'+MM+'-'+dd+' '+hh+':'+mm+':'+ss;
        case 'mm/dd/yyyy hh:mm:ss':
          return MM+'/'+dd+'/'+yyyy+' '+hh+':'+mm+':'+ss;
        case 'yyyy-mm-ddthh:mm':
          return yyyy+'-'+MM+'-'+dd+'T'+hh+':'+mm;
        case 'yyyy-mm-dd hh:mm':
          return yyyy+'-'+MM+'-'+dd+' '+hh+':'+mm;
        case 'mm/dd/yyyy hh:mm':
          return MM+'/'+dd+'/'+yyyy+' '+hh+':'+mm;
        case 'yyyy-mm-dd':
          return yyyy+'-'+MM+'-'+dd;
        case 'mm/dd/yyyy':
          return MM+'/'+dd+'/'+yyyy
        case 'mm/dd':
          return MM+'/'+dd;
        case 'dd/mm':
          return dd+'/'+MM;
        case 'hh:mm:ss.fff':
          return hh+':'+mm+':'+ss+'.'+fff;
        case 'hh:mm:ss':
          return hh+':'+mm+':'+ss;
        case 'hh:mm':
          return hh+':'+mm;
        default:
          return date.toString();
      }
    };
    Nlib.prototype.timeOffset = function(unit) {
      var now = new Date();
      var local = (new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0, 0)).getTime();
      var utc = Date.UTC(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0, 0);
      var offset = utc - local;
      switch (unit) {
        case 'seconds':
          return offset / 1000;
        case 'minutes':
          return offset / 60 * 1000;
        case 'hours':
          return offset / 60 * 60 * 1000;
        default:
          return offset;
      }
    };
  })();

  // **************************************************************************
  // Hash
  // **************************************************************************
  (function() {
    var crc32 = null;
    var crc32table = function() {
      if (!crc32) {
        var c;
        var crcTable = [];
        for (var n = 0; n < 256; ++n){
          c = n;
          for(var k =0; k < 8; k++){
            c = ((c&1) ? (0xEDB88320 ^ (c >>> 1)) : (c >>> 1));
          }
          crcTable[n] = c;
        }
        crc32 = crcTable;
      }
      return crc32;
    };
    Nlib.prototype.crc32 = function(data, crc) {
      if (crc !== undefined)  crc = 0 ^ (-1);
      var i;
      var l;

      var table = crc32table();
      if (typeof data === 'string') {
        var str = data;
        var c;
        for (i = 0, l = str.length; i < l; ++i) {
          c = str.charCodeAt(i);
          if (c < 0x80) {
            crc = (crc >>> 8) ^ table[(crc ^ c) & 0xFF];
          } else if(c < 0x800) {
            crc = (crc >>> 8) ^ table[(crc ^ (192|((c>>6)&31))) & 0xFF];
            crc = (crc >>> 8) ^ table[(crc ^ (128|(c&63))) & 0xFF];
          } else if (c >= 0xD800 && c < 0xE000) {
            c = (c&1023)+64; d = str.charCodeAt(i++) & 1023;
            crc = (crc >>> 8) ^ table[(crc ^ (240|((c>>8)&7))) & 0xFF];
            crc = (crc >>> 8) ^ table[(crc ^ (128|((c>>2)&63))) & 0xFF];
            crc = (crc >>> 8) ^ table[(crc ^ (128|((d>>6)&15)|(c&3))) & 0xFF];
            crc = (crc >>> 8) ^ table[(crc ^ (128|(d&63))) & 0xFF];
          } else {
            crc = (crc >>> 8) ^ table[(crc ^ (224|((c>>12)&15))) & 0xFF];
            crc = (crc >>> 8) ^ table[(crc ^ (128|((c>>6)&63))) & 0xFF];
            crc = (crc >>> 8) ^ table[(crc ^ (128|(c&63))) & 0xFF];
          }
        }
        return (crc ^ -1) >>> 0;
      } else if (typeof module !== 'undefined' && module.exports) {
        var Buffer = require('buffer').Buffer;
        var buf = Buffer.isBuffer(data) ? data : Buffer(data);
        if (nodejs.Buffer.isBuffer(crc)) {
          crc = crc.readUInt32BE(0);
        }
        crc = ~~crc ^ -1;
        for (i = 0, l = buf.length; i < l; ++i) {
          crc = (crc >>> 8) ^ table[(crc ^ buf[i]) & 0xff];
        }
        return (crc ^ -1) >>> 0;
      }
      return undefined;
    };
  })();

  // **************************************************************************
  // Object
  // **************************************************************************
  (function() {
    Nlib.prototype.isFunction = function(object) {
      return !!(object && object.constructor && object.call && object.apply);
    };
    Nlib.prototype.isArray = function(object) {
      return (object && object.constructor == Array);
    };
    Nlib.prototype.forEach = function(items, callback) {
      var self = this;
      if (items) {
        if (typeof items == 'object') {
          if (self.isArray(items)) {
            items.forEach(callback);
          } else {
            for (var key in items) {
              if (items.hasOwnProperty(key)) {
                callback(items[key], key);
              }
            }
          }
        } else {
          callback(items);
        }
      }
    };
    Nlib.prototype.every = function(items, callback) {
      if (items) {
        if (typeof items == 'object') {
          if (items.constructor == Array) {
            return items.every(callback);
          } else {
            for (var key in items) {
              if (items.hasOwnProperty(key)) {
                if (!callback(items[key], key)) {
                  return false;
                }
              }
            }
            return true;
          }
        } else {
          return callback(items);
        }
      }
      return true;
    };
    Nlib.prototype.array = function(items) {
      if (items) {
        var array = [];
        if (typeof items === 'object') {
          if (items.constructor == Array) {
            return items;
          } else {
            for (var i in items) {
              if (items.hasOwnProperty(i)) {
                array.push(items[i]);
              }
            }
          }
        } else {
          array.push(items);
        }
        return array;
      }
      return null;
    };
    Nlib.prototype.clone = function(item, deep) {
      var self = this;
      if (typeof item === 'object') {
        var clone;
        if (item.constructor == Array) {
          clone = [];
          for (var i = 0; i < item.length; ++i) {
            clone[i] = (deep)
              ? self.clone(item[i], deep)
              : item[i];
          }
        } else {
          clone = { };
          for (var name in item) {
            if (item.hasOwnProperty(name)) {
              clone[name] = (deep)
                ? self.clone(item[name], deep)
                : item[name];
            }
          }
        }
        return clone;
      } else {
        return item;
      }
    };
    Nlib.prototype.update = function(item, update) {
      var self = this;
      var hasChanges = false;
      this.forEach(update, function(value, key) {
        if (value !== undefined) {
          if (item[key] !== undefined) {
            if (typeof value === 'object') {
              hasChanges = this.update(item[key], value) || hasChanges;
            } else if (item[key] != value) {
              hasChanges = true;
              item[key] = value;
            }
          } else {
            hasChanges = true;
            item[key] = value;
          }
        }
      });
      return hasChanges;
    };
    Nlib.prototype.equal = function(item1, item2) {
      var self = this;
      var result = true;
      if (typeof item1 == 'object' && typeof item2 == 'object') {
        if (item1.constructor == Array && item2.constructor == Array) {
          result = result && (item1.length == item2.length);
          result = result &&
            (
              self.every(item1, function(value1) {
                return !self.every(item2, function(value2) {
                  return !self.equal(value1, value2);
                });
              })
            );
        } else {
          result = result &&
            self.every(item1, function(value, key) {
              return item2.hasOwnProperty(key) && self.equal(value, item2[key]);
            });
          result = result &&
            self.every(item2, function(value, key) {
              return item1.hasOwnProperty(key) && self.equal(value, item1[key]);
            });
        }
      } else {
        result = result && (item1 === item2);
      }
      return result;
    };
    self.match = function(item, query) {
      return self.every(query, function(value, name) {
        return (name in item) && self.equal(item[name], value);
      });
    };
  })();

  // **************************************************************************
  // Configuration (NodeJS only)
  // **************************************************************************
  (function() {
    if (typeof module == 'undefined' || !module.exports) {
      return;
    }
    var config = null;
    Nlib.prototype.loadConfig = function(location, callback) {
      var env = process.env['NODE_ENV'] || 'development';
      var path = require('path').join(location, '/' + env + '.json');
      fs.readFile(path, { encoding: 'utf8'}, function(err, data) {
        if (err) {
          callback(err);
        } else {
          config = JSON.parse(data);
        }
      });
    };
    Nlib.prototype.config = function() {
      return config;
    };
  })();

  // **************************************************************************
  // PLATFORM
  // **************************************************************************
  if (typeof module !== 'undefined' && module.exports) {
    module.exports = new Nlib();
  } else {
    global.nlib = new Nlib();
  }

}(this));